==> vm-0.10.0.tgz: package/lib/vm.js <==
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _VM_globalCache, _VM_localCache;
import { getQuickJS, isFail } from "quickjs-emscripten";
import { bytesToHex } from "@noble/hashes/utils";
import { sha256 } from "@noble/hashes/sha256";
import { logger } from "@libp2p/logger";
import { assert, mapValues } from "@canvas-js/utils";
import { VMError } from "./error.js";
export class VM {
    static async initialize(options = {}) {
        const quickJS = await getQuickJS();
        const runtime = quickJS.newRuntime();
        const context = runtime.newContext();
        return new VM(runtime, context, options);
    }
    constructor(runtime, context, options) {
        this.runtime = runtime;
        this.context = context;
        this.log = logger("canvas:vm");
        _VM_globalCache.set(this, new Map());
        _VM_localCache.set(this, new Set());
        this.get = (path) => {
            const cachedHandle = __classPrivateFieldGet(this, _VM_globalCache, "f").get(path);
            if (cachedHandle !== undefined) {
                return cachedHandle;
            }
            const elements = path.split(".");
            const prop = elements.pop();
            assert(prop !== undefined);
            const object = elements.length > 0 ? this.get(elements.join(".")) : this.context.global;
            const handle = this.context.getProp(object, prop);
            __classPrivateFieldGet(this, _VM_globalCache, "f").set(path, handle);
            return handle;
        };
        /**
         * Resolving promises inside QuickJS is tricky because you have to call
         * runtime.executePendingJobs() to get the promise to resolve, so you
         * can't use await syntax even though context.resolvePromise returns a
         * native Promise. This is a utility method that lets you use await.
         */
        this.resolvePromise = (promise) => {
            return new Promise((resolve, reject) => {
                this.context.resolvePromise(promise).then((result) => {
                    if (isFail(result)) {
                        reject(result.error.consume(this.unwrapError));
                    }
                    else {
                        resolve(result.value);
                    }
                }, reject);
                this.runtime.executePendingJobs();
            });
        };
        this.call = (fn, thisArg, args) => {
            const result = this.context.callFunction(typeof fn === "string" ? this.get(fn) : fn, typeof thisArg === "string" ? this.get(thisArg) : thisArg, ...args);
            return this.unwrapResult(result);
        };
        this.callAsync = async (fn, thisArg, args) => {
            const resultHandle = this.call(fn, thisArg, args);
            if (this.isInstanceOf(resultHandle, this.get("Promise"))) {
                return resultHandle.consume(this.resolvePromise);
            }
            else {
                return resultHandle;
            }
        };
        this.unwrapError = (handle) => {
            return new VMError(this.context.dump(handle));
        };
        this.wrapError = (err) => {
            const error = err instanceof Error ? err : new Error(String(err));
            let constructorName = "Error";
            if (error instanceof SyntaxError) {
                constructorName = "SyntaxError";
            }
            else if (error instanceof TypeError) {
                constructorName = "TypeError";
            }
            else if (error instanceof RangeError) {
                constructorName = "RangeError";
            }
            const args = [this.context.newString(error.message)];
            try {
                const errorHandle = this.call(constructorName, this.context.null, args);
                if (error.stack !== undefined) {
                    this.context.setProp(errorHandle, "stack", this.context.newString(error.stack));
                }
                return errorHandle;
            }
            finally {
                args.forEach((arg) => arg.dispose());
            }
        };
        this.unwrapResult = (result) => {
            if (isFail(result)) {
                throw result.error.consume(this.unwrapError);
            }
            else {
                return result.value;
            }
        };
        this.getBoolean = (handle) => Boolean(this.context.getNumber(handle));
        this.getUint8Array = (handle) => Uint8Array.from(this.call("Array.from", "Array", [handle]).consume(this.context.dump));
        this.newUint8Array = (value) => {
            const byteArray = Array.from(value).map((byte) => this.context.newNumber(byte));
            return this.wrapArray(byteArray).consume((handle) => this.call("Uint8Array.from", "Uint8Array", [handle]));
        };
        /**
         * Wrap an array outside a QuickJS VM by one level,
         * returning a QuickJSHandle in the host environment.
         * `wrapArray` disposes all of its handle elements.
         */
        this.wrapArray = (array) => {
            const arrayHandle = this.context.newArray();
            for (const elementHandle of array) {
                this.call("Array.prototype.push", arrayHandle, [elementHandle]).dispose();
                elementHandle.dispose();
            }
            return arrayHandle;
        };
        /**
         * Unwrap an array inside a QuickJS VM by one level,
         * returning a QuickJSHandle[] in the host environment.
         * `unwrapArray` does NOT dispose of the original handle.
         */
        this.unwrapArray = (handle, map) => {
            const length = this.context.getProp(handle, "length").consume(this.context.getNumber);
            const array = new Array(length);
            for (let index = 0; index < length; index++) {
                const elementHandle = this.context
                    .newNumber(index)
                    .consume((indexHandle) => this.context.getProp(handle, indexHandle));
                if (map === undefined) {
                    array[index] = elementHandle;
                }
                else {
                    array[index] = map(elementHandle);
                }
            }
            return array;
        };
        /**
         * Wrap an object outside a QuickJS VM by one level,
         * returning a QuickJSHandle in the host environment.
         * `wrapObject` disposes all of its handle values.
         */
        this.wrapObject = (object) => {
            const objectHandle = this.context.newObject();
            for (const [key, valueHandle] of Object.entries(object)) {
                valueHandle.consume((handle) => this.context.setProp(objectHandle, key, handle));
            }
            return objectHandle;
        };
        /**
         * Unwrap an object inside a QuickJS VM by one level,
         * returning a Record<string, QuickJSHandle> in the host environment.
         * `unwrapObject` does NOT dispose of the original handle.
         */
        this.unwrapObject = (handle, map) => {
            const object = {};
            const keys = this.call("Object.keys", this.context.null, [handle]).consume(this.unwrapArray);
            for (const keyHandle of keys) {
                const valueHandle = this.context.getProp(handle, keyHandle);
                const key = keyHandle.consume(this.context.getString);
                if (map === undefined) {
                    object[key] = valueHandle;
                }
                else {
                    object[key] = map(valueHandle);
                }
            }
            return object;
        };
        this.wrapValue = (value) => {
            if (value === undefined) {
                return this.context.undefined;
            }
            else if (value === null) {
                return this.context.null;
            }
            else if (typeof value === "boolean") {
                return value ? this.context.true : this.context.false;
            }
            else if (typeof value === "number") {
                return this.context.newNumber(value);
            }
            else if (typeof value === "string") {
                return this.context.newString(value);
            }
            else if (value instanceof Uint8Array) {
                return this.newUint8Array(value);
            }
            else if (Array.isArray(value)) {
                return this.wrapArray(value.map(this.wrapValue));
            }
            else {
                return this.wrapObject(mapValues(value, this.wrapValue));
            }
        };
        this.is = (a, b) => this.call("Object.is", this.context.null, [a, b]).consume(this.context.dump);
        this.isArray = (handle) => this.context.typeof(handle) === "object" &&
            this.call("Array.isArray", this.context.null, [handle]).consume(this.context.dump);
        this.isInstanceOf = (instanceHandle, classHandle) => this.context.typeof(instanceHandle) === "object" &&
            this.context
                .getProp(instanceHandle, "constructor")
                .consume((constructorHandle) => this.is(constructorHandle, classHandle));
        this.isUint8Array = (handle) => this.isInstanceOf(handle, this.get("Uint8Array"));
        this.unwrapValue = (handle) => {
            if (this.is(handle, this.context.undefined)) {
                throw new Error("cannot unwrap `undefined`");
            }
            else if (this.is(handle, this.context.null)) {
                return null;
            }
            else if (this.context.typeof(handle) === "boolean") {
                return this.getBoolean(handle);
            }
            else if (this.context.typeof(handle) === "number") {
                return this.context.getNumber(handle);
            }
            else if (this.context.typeof(handle) === "string") {
                return this.context.getString(handle);
            }
            else if (this.isUint8Array(handle)) {
                return this.getUint8Array(handle);
            }
            else if (this.isArray(handle)) {
                return this.unwrapArray(handle, (elementHandle) => elementHandle.consume(this.unwrapValue));
            }
            else {
                return this.unwrapObject(handle, (propertyHandle) => propertyHandle.consume(this.unwrapValue));
            }
        };
        this.wrapFunction = (fn) => {
            const wrap = (value) => (value === undefined ? this.context.undefined : this.wrapValue(value));
            return this.context.newFunction(fn.name, (...args) => {
                let result = undefined;
                this.log("invoking function");
                try {
                    result = fn(...args.map(this.unwrapValue));
                }
                catch (err) {
                    this.log("caught error in wrapped function: %O", err);
                    return this.wrapError(err);
                }
                this.log("got result");
                if (result instanceof Promise) {
                    const deferred = this.context.newPromise();
                    result.then((value) => {
                        this.log("resolving deferred promise");
                        const handle = wrap(value);
                        deferred.settled.then(() => handle.dispose());
                        deferred.resolve(handle);
                        // wrap(value).consume((handle) => deferred.resolve(handle))
                    }, (err) => {
                        this.log("rejecting deferred promise");
                        deferred.reject(this.wrapError(err));
                    });
                    deferred.settled.finally(() => this.runtime.executePendingJobs());
                    return deferred.handle;
                }
                else {
                    return wrap(result);
                }
            });
        };
        this.unwrapFunction = (handle, thisArg) => {
            const copy = handle.dup();
            __classPrivateFieldGet(this, _VM_localCache, "f").add(copy);
            let thisArgCopy = copy;
            if (thisArg !== undefined) {
                thisArgCopy = thisArg.dup();
                __classPrivateFieldGet(this, _VM_localCache, "f").add(thisArgCopy);
            }
            return (...args) => {
                const argHandles = args.map(this.wrapValue);
                try {
                    const result = this.call(copy, thisArgCopy, argHandles);
                    if (this.context.typeof(result) !== "undefined") {
                        return result.consume(this.unwrapValue);
                    }
                }
                finally {
                    argHandles.forEach((handle) => handle.dispose());
                }
            };
        };
        this.unwrapFunctionAsync = (handle, thisArg) => {
            const copy = this.cache(handle);
            const thisArgCopy = thisArg ? this.cache(thisArg) : copy;
            return async (...args) => {
                const argHandles = args.map(this.wrapValue);
                try {
                    const result = await this.callAsync(copy, thisArgCopy, argHandles);
                    if (this.context.typeof(result) !== "undefined") {
                        return result.consume(this.unwrapValue);
                    }
                }
                finally {
                    argHandles.forEach((handle) => handle.dispose());
                }
            };
        };
        this.cache = (handle) => {
            const copy = handle.dup();
            __classPrivateFieldGet(this, _VM_localCache, "f").add(copy);
            return copy;
        };
        this.runtime.setMemoryLimit(options.runtimeMemoryLimit ?? VM.RUNTIME_MEMORY_LIMIT);
        const log = options.log ?? ((...args) => console.log("[vm]", ...args));
        this.setGlobalValues({
            console: this.wrapObject({ log: this.wrapFunction((...args) => void log(...args)) }),
        });
    }
    /**
     * Cleans up this VM instance.
     */
    dispose() {
        for (const [path, handle] of __classPrivateFieldGet(this, _VM_globalCache, "f")) {
            handle.dispose();
            __classPrivateFieldGet(this, _VM_globalCache, "f").delete(path);
        }
        for (const handle of __classPrivateFieldGet(this, _VM_localCache, "f")) {
            handle.dispose();
            __classPrivateFieldGet(this, _VM_localCache, "f").delete(handle);
        }
        this.context.dispose();
        this.runtime.dispose();
    }
    setGlobalValues(values) {
        for (const [name, handle] of Object.entries(values)) {
            handle.consume((handle) => this.context.setProp(this.context.global, name, handle));
        }
    }
    execute(contract, options = {}) {
        const filename = options.uri ?? `canvas:${bytesToHex(sha256(contract))}`;
        this.unwrapResult(this.context.evalCode(contract, filename, { type: "global", strict: true })).dispose();
    }
    async import(contract, options = {}) {
        const filename = options.uri ?? `canvas:${bytesToHex(sha256(contract))}`;
        this.runtime.setModuleLoader((moduleName) => {
            if (moduleName === filename) {
                return contract;
            }
            else {
                return { error: new Error("module not found") };
            }
        });
        try {
            const promiseHandle = this.unwrapResult(this.context.evalCode(`import("${filename}")`, undefined, { type: "global", strict: true }));
            const exportsHandle = await promiseHandle.consume(this.resolvePromise);
            return exportsHandle;
        }
        finally {
            this.runtime.removeModuleLoader();
        }
    }
}
_VM_globalCache = new WeakMap(), _VM_localCache = new WeakMap();
VM.RUNTIME_MEMORY_LIMIT = 1024 * 640; // 640kb

==> vm-0.10.6.tgz: package/lib/vm.js <==
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _VM_globalCache, _VM_localCache;
import { getQuickJS, isFail } from "quickjs-emscripten";
import { bytesToHex } from "@noble/hashes/utils";
import { sha256 } from "@noble/hashes/sha256";
import { logger } from "@libp2p/logger";
import { assert, mapValues } from "@canvas-js/utils";
import { VMError } from "./error.js";
export class VM {
    static async initialize(options = {}) {
        const quickJS = await getQuickJS();
        const runtime = quickJS.newRuntime();
        const context = runtime.newContext();
        return new VM(runtime, context, options);
    }
    constructor(runtime, context, options) {
        this.runtime = runtime;
        this.context = context;
        this.log = logger("canvas:vm");
        _VM_globalCache.set(this, new Map());
        _VM_localCache.set(this, new Set());
        this.get = (path) => {
            const cachedHandle = __classPrivateFieldGet(this, _VM_globalCache, "f").get(path);
            if (cachedHandle !== undefined) {
                return cachedHandle;
            }
            const elements = path.split(".");
            const prop = elements.pop();
            assert(prop !== undefined);
            const object = elements.length > 0 ? this.get(elements.join(".")) : this.context.global;
            const handle = this.context.getProp(object, prop);
            __classPrivateFieldGet(this, _VM_globalCache, "f").set(path, handle);
            return handle;
        };
        /**
         * Resolving promises inside QuickJS is tricky because you have to call
         * runtime.executePendingJobs() to get the promise to resolve, so you
         * can't use await syntax even though context.resolvePromise returns a
         * native Promise. This is a utility method that lets you use await.
         */
        this.resolvePromise = (promise) => {
            return new Promise((resolve, reject) => {
                this.context.resolvePromise(promise).then((result) => {
                    if (isFail(result)) {
                        reject(result.error.consume(this.unwrapError));
                    }
                    else {
                        resolve(result.value);
                    }
                }, reject);
                this.runtime.executePendingJobs();
            });
        };
        this.call = (fn, thisArg, args) => {
            const result = this.context.callFunction(typeof fn === "string" ? this.get(fn) : fn, typeof thisArg === "string" ? this.get(thisArg) : thisArg, ...args);
            return this.unwrapResult(result);
        };
        this.callAsync = async (fn, thisArg, args) => {
            const resultHandle = this.call(fn, thisArg, args);
            if (this.isInstanceOf(resultHandle, this.get("Promise"))) {
                return resultHandle.consume(this.resolvePromise);
            }
            else {
                return resultHandle;
            }
        };
        this.unwrapError = (handle) => {
            return new VMError(this.context.dump(handle));
        };
        this.wrapError = (err) => {
            const error = err instanceof Error ? err : new Error(String(err));
            let constructorName = "Error";
            if (error instanceof SyntaxError) {
                constructorName = "SyntaxError";
            }
            else if (error instanceof TypeError) {
                constructorName = "TypeError";
            }
            else if (error instanceof RangeError) {
                constructorName = "RangeError";
            }
            const args = [this.context.newString(error.message)];
            try {
                const errorHandle = this.call(constructorName, this.context.null, args);
                if (error.stack !== undefined) {
                    this.context.setProp(errorHandle, "stack", this.context.newString(error.stack));
                }
                return errorHandle;
            }
            finally {
                args.forEach((arg) => arg.dispose());
            }
        };
        this.unwrapResult = (result) => {
            if (isFail(result)) {
                throw result.error.consume(this.unwrapError);
            }
            else {
                return result.value;
            }
        };
        this.getBoolean = (handle) => Boolean(this.context.getNumber(handle));
        this.getUint8Array = (handle) => Uint8Array.from(this.call("Array.from", "Array", [handle]).consume(this.context.dump));
        this.newUint8Array = (value) => {
            const byteArray = Array.from(value).map((byte) => this.context.newNumber(byte));
            return this.wrapArray(byteArray).consume((handle) => this.call("Uint8Array.from", "Uint8Array", [handle]));
        };
        /**
         * Wrap an array outside a QuickJS VM by one level,
         * returning a QuickJSHandle in the host environment.
         * `wrapArray` disposes all of its handle elements.
         */
        this.wrapArray = (array) => {
            const arrayHandle = this.context.newArray();
            for (const elementHandle of array) {
                this.call("Array.prototype.push", arrayHandle, [elementHandle]).dispose();
                elementHandle.dispose();
            }
            return arrayHandle;
        };
        /**
         * Unwrap an array inside a QuickJS VM by one level,
         * returning a QuickJSHandle[] in the host environment.
         * `unwrapArray` does NOT dispose of the original handle.
         */
        this.unwrapArray = (handle, map) => {
            const length = this.context.getProp(handle, "length").consume(this.context.getNumber);
            const array = new Array(length);
            for (let index = 0; index < length; index++) {
                const elementHandle = this.context
                    .newNumber(index)
                    .consume((indexHandle) => this.context.getProp(handle, indexHandle));
                if (map === undefined) {
                    array[index] = elementHandle;
                }
                else {
                    array[index] = map(elementHandle);
                }
            }
            return array;
        };
        /**
         * Wrap an object outside a QuickJS VM by one level,
         * returning a QuickJSHandle in the host environment.
         * `wrapObject` disposes all of its handle values.
         */
        this.wrapObject = (object) => {
            const objectHandle = this.context.newObject();
            for (const [key, valueHandle] of Object.entries(object)) {
                valueHandle.consume((handle) => this.context.setProp(objectHandle, key, handle));
            }
            return objectHandle;
        };
        /**
         * Unwrap an object inside a QuickJS VM by one level,
         * returning a Record<string, QuickJSHandle> in the host environment.
         * `unwrapObject` does NOT dispose of the original handle.
         */
        this.unwrapObject = (handle, map) => {
            const object = {};
            const keys = this.call("Object.keys", this.context.null, [handle]).consume(this.unwrapArray);
            for (const keyHandle of keys) {
                const valueHandle = this.context.getProp(handle, keyHandle);
                const key = keyHandle.consume(this.context.getString);
                if (map === undefined) {
                    object[key] = valueHandle;
                }
                else {
                    object[key] = map(valueHandle);
                }
            }
            return object;
        };
        this.wrapValue = (value) => {
            if (value === undefined) {
                return this.context.undefined;
            }
            else if (value === null) {
                return this.context.null;
            }
            else if (typeof value === "boolean") {
                return value ? this.context.true : this.context.false;
            }
            else if (typeof value === "number") {
                return this.context.newNumber(value);
            }
            else if (typeof value === "string") {
                return this.context.newString(value);
            }
            else if (value instanceof Uint8Array) {
                return this.newUint8Array(value);
            }
            else if (Array.isArray(value)) {
                return this.wrapArray(value.map(this.wrapValue));
            }
            else {
                return this.wrapObject(mapValues(value, this.wrapValue));
            }
        };
        this.is = (a, b) => this.call("Object.is", this.context.null, [a, b]).consume(this.context.dump);
        this.isArray = (handle) => this.context.typeof(handle) === "object" &&
            this.call("Array.isArray", this.context.null, [handle]).consume(this.context.dump);
        this.isInstanceOf = (instanceHandle, classHandle) => this.context.typeof(instanceHandle) === "object" &&
            this.context
                .getProp(instanceHandle, "constructor")
                .consume((constructorHandle) => this.is(constructorHandle, classHandle));
        this.isUint8Array = (handle) => this.isInstanceOf(handle, this.get("Uint8Array"));
        this.unwrapValue = (handle) => {
            if (this.is(handle, this.context.undefined)) {
                throw new Error("cannot unwrap `undefined`");
            }
            else if (this.is(handle, this.context.null)) {
                return null;
            }
            else if (this.context.typeof(handle) === "boolean") {
                return this.getBoolean(handle);
            }
            else if (this.context.typeof(handle) === "number") {
                return this.context.getNumber(handle);
            }
            else if (this.context.typeof(handle) === "string") {
                return this.context.getString(handle);
            }
            else if (this.isUint8Array(handle)) {
                return this.getUint8Array(handle);
            }
            else if (this.isArray(handle)) {
                return this.unwrapArray(handle, (elementHandle) => elementHandle.consume(this.unwrapValue));
            }
            else {
                return this.unwrapObject(handle, (propertyHandle) => propertyHandle.consume(this.unwrapValue));
            }
        };
        this.wrapFunction = (fn) => {
            const wrap = (value) => (value === undefined ? this.context.undefined : this.wrapValue(value));
            return this.context.newFunction(fn.name, (...args) => {
                let result = undefined;
                this.log("invoking function");
                try {
                    result = fn(...args.map(this.unwrapValue));
                }
                catch (err) {
                    this.log("caught error in wrapped function: %O", err);
                    return this.wrapError(err);
                }
                this.log("got result");
                if (result instanceof Promise) {
                    const deferred = this.context.newPromise();
                    result.then((value) => {
                        this.log("resolving deferred promise");
                        const handle = wrap(value);
                        deferred.settled.then(() => handle.dispose());
                        deferred.resolve(handle);
                        // wrap(value).consume((handle) => deferred.resolve(handle))
                    }, (err) => {
                        this.log("rejecting deferred promise");
                        deferred.reject(this.wrapError(err));
                    });
                    deferred.settled.finally(() => this.runtime.executePendingJobs());
                    return deferred.handle;
                }
                else {
                    return wrap(result);
                }
            });
        };
        this.unwrapFunction = (handle, thisArg) => {
            const copy = handle.dup();
            __classPrivateFieldGet(this, _VM_localCache, "f").add(copy);
            let thisArgCopy = copy;
            if (thisArg !== undefined) {
                thisArgCopy = thisArg.dup();
                __classPrivateFieldGet(this, _VM_localCache, "f").add(thisArgCopy);
            }
            return (...args) => {
                const argHandles = args.map(this.wrapValue);
                try {
                    const result = this.call(copy, thisArgCopy, argHandles);
                    if (this.context.typeof(result) !== "undefined") {
                        return result.consume(this.unwrapValue);
                    }
                }
                finally {
                    argHandles.forEach((handle) => handle.dispose());
                }
            };
        };
        this.unwrapFunctionAsync = (handle, thisArg) => {
            const copy = this.cache(handle);
            const thisArgCopy = thisArg ? this.cache(thisArg) : copy;
            return async (...args) => {
                const argHandles = args.map(this.wrapValue);
                try {
                    const result = await this.callAsync(copy, thisArgCopy, argHandles);
                    if (this.context.typeof(result) !== "undefined") {
                        return result.consume(this.unwrapValue);
                    }
                }
                finally {
                    argHandles.forEach((handle) => handle.dispose());
                }
            };
        };
        this.cache = (handle) => {
            const copy = handle.dup();
            __classPrivateFieldGet(this, _VM_localCache, "f").add(copy);
            return copy;
        };
        this.runtime.setMemoryLimit(options.runtimeMemoryLimit ?? VM.RUNTIME_MEMORY_LIMIT);
        const log = options.log ?? ((...args) => console.log("[vm]", ...args));
        this.setGlobalValues({
            console: this.wrapObject({ log: this.wrapFunction((...args) => void log(...args)) }),
        });
    }
    /**
     * Cleans up this VM instance.
     */
    dispose() {
        for (const [path, handle] of __classPrivateFieldGet(this, _VM_globalCache, "f")) {
            handle.dispose();
            __classPrivateFieldGet(this, _VM_globalCache, "f").delete(path);
        }
        for (const handle of __classPrivateFieldGet(this, _VM_localCache, "f")) {
            handle.dispose();
            __classPrivateFieldGet(this, _VM_localCache, "f").delete(handle);
        }
        this.context.dispose();
        this.runtime.dispose();
    }
    setGlobalValues(values) {
        for (const [name, handle] of Object.entries(values)) {
            handle.consume((handle) => this.context.setProp(this.context.global, name, handle));
        }
    }
    execute(contract, options = {}) {
        const filename = options.uri ?? `canvas:${bytesToHex(sha256(contract))}`;
        this.unwrapResult(this.context.evalCode(contract, filename, { type: "global", strict: true })).dispose();
    }
    import(contract, options = {}) {
        const filename = options.uri ?? `canvas:${bytesToHex(sha256(contract))}`;
        return this.unwrapResult(this.context.evalCode(contract, filename, { type: "module" }));
    }
}
_VM_globalCache = new WeakMap(), _VM_localCache = new WeakMap();
VM.RUNTIME_MEMORY_LIMIT = 1024 * 640; // 640kb
